package org.jeecg.module.custom.ocr.utils;

import org.apache.commons.lang3.time.DateFormatUtils;
import org.jeecg.module.custom.ocr.base.constant.SystemConstants;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @author zhangyong
 * @title: SnowFlakeUtil
 * @projectName jyjz-parent
 * @description: 时间工具类
 * @date 2021/11/13 10:06 AM
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils
{
    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";

    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    
    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};

    /**
     * 获取当前Date型日期
     * 
     * @return Date() 当前日期
     */
    public static Date getNowDate()
    {
        return new Date();
    }
    public static Date getNowDateYmd()
    {
        String date = parseDateToStr(YYYY_MM_DD,new Date());
        return dateTime(YYYY_MM_DD,date);
    }
    public static Date getNextDateYmd(){
        //创建日历对象
        Calendar calendar = Calendar.getInstance();
        //得倒今天的日期
        calendar.setTime(new Date());
        //计算昨天的日期
        calendar.add(calendar.DATE,+ 1);
        //格式化
        String date = parseDateToStr(YYYY_MM_DD,calendar.getTime());
        return dateTime(YYYY_MM_DD,date);
    }
    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     * 
     * @return String
     */
    public static String getDate()
    {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime()
    {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow()
    {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format)
    {
        return parseDateToStr(format, new Date());
    }
    /**
     * long转换成字符串日期
     * @param longTime 日期
     * @return yyyy-MM-dd HH:mm:ss
     */
    public static final String getDateTime(long longTime) {
        return parseDateToStr(YYYY_MM_DD_HH_MM_SS,new Date(longTime));
    }
    
    /**
     * 获取当前时间的时间戳，返回秒级时间戳
     * @return
     */
    public static Long getSecondTimestamp(){
        return System.currentTimeMillis()/1000;
    }
    /**
     * 毫秒时间戳转换为秒级时间戳
     * @return
     */
    public static Long getSecondTimestamp(Long time){
        return time/1000;
    }
    
    /**
     * long转换成字符串日期
     * @param longTime 日期
     * @return yyyy-MM-dd
     */
    public static final String getDate(long longTime) {
        return parseDateToStr(YYYY_MM_DD,new Date(longTime));
    }
    public static final String dateTime(final Date date)
    {
        return parseDateToStr(YYYY_MM_DD, date);
    }
    public static final String parseDateToStr(long longTime)
    {
        return parseDateToStr(YYYY_MM_DD,new Date(longTime));
    }
    public static final String parseDateTimeToStr(long longTime)
    {
        return parseDateToStr(YYYY_MM_DD_HH_MM_SS,new Date(longTime));
    }
    public static final String parseDateToStr(final String format, final Date date)
    {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts)
    {
        try
        {
            return new SimpleDateFormat(format).parse(ts);
        }
        catch (ParseException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str)
    {
        if (str == null)
        {
            return null;
        }
        try
        {
            return parseDate(str.toString(), parsePatterns);
        }
        catch (ParseException e)
        {
            return null;
        }
    }
    
    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate()
    {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(Date date1, Date date2)
    {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
    }

    /**
     * 计算两个时间差
     */
    public static String getDatePoor(Date endDate, Date nowDate)
    {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - nowDate.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }
    public static boolean isValidDateTime(String strDate) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            format.parse(strDate);
        } catch (Exception e) {
            // e.printStackTrace();
            // 如果throw java.text.ParseException或者NullPointerException，就说明格式不对
            return false;
        }
        
        return true;
    }
    public static boolean isValidDate(String strDate) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        try {
            format.parse(strDate);
        } catch (Exception e) {
            // e.printStackTrace();
            // 如果throw java.text.ParseException或者NullPointerException，就说明格式不对
            return false;
        }
        
        return true;
    }
    /**
     * 判断两个日期是否为同一天
     * @param endDate
     * @param nowDate
     * @return
     */
    public static boolean sameDay(Long endDate, Long nowDate)
    {
        String tempEnd = DateUtils.parseDateToStr(endDate);
        String tempNow = DateUtils.parseDateToStr(nowDate);
        
        return tempNow.equals(tempEnd);
    }
    
    /**
     * 减去一段时间后，是否还是当天时间
     * @param thisDateEnd 当前时间，毫秒
     * @param subtime 被减去的时间，毫秒
     * @return 如果是当天时间返回true，否则返回false
     */
    public static boolean validateThisDate(long thisDateEnd,long subtime){
        //判断当前时间是否能存在跨天情况
        String thisDateStr = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, new Date(thisDateEnd));
    
        //减去15分钟后的时间
        Long searchStartTime = thisDateEnd - subtime;
    
        //时间字符串,年月日
        String startStr = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, new Date(searchStartTime));
    
        //如果两个时间不等，说明不是同一天，需要重新查询
        if(startStr.equals(thisDateStr)){
            return true;
        }
        return false;
    }
    public static void main(String argss[]){
        long thistime = System.currentTimeMillis();
        long subtime = SystemConstants.SYS_ROOT_TIME_DVALUE * 6000 * 1000;
        System.out.println(DateUtils.validateThisDate(thistime,subtime));
//        String tim = "2023-2-18 13:30:47";
//
//        System.out.println(DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,tim).getTime());
//        Date date = new Date(1676698247000L);
//        System.out.println(DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,tim).getTime());
    }
}
